# Copyright (C) 2014-2016 Andrey Antukh <niwi@niwi.nz>
# Copyright (C) 2014-2016 Jesús Espino <jespinog@gmail.com>
# Copyright (C) 2014-2016 David Barragán <bameda@dbarragan.com>
# Copyright (C) 2014-2016 Alejandro Alonso <alejandro.alonso@kaleidos.net>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.apps import AppConfig


class TaigaContribGitlabAuthAppConfig(AppConfig):
    name = "taiga_contrib_gitlab_auth"
    verbose_name = "Taiga GitLab Authentication App Config"

    def ready(self):
        from taiga.auth.services import register_auth_plugin
        from . import services
        register_auth_plugin("gitlab", services.gitlab_login_func)




# Copyright (C) 2014 Andrey Antukh <niwi@niwi.be>
# Copyright (C) 2014 Jesús Espino <jespinog@gmail.com>
# Copyright (C) 2014 David Barragán <bameda@dbarragan.com>
# Copyright (C) 2014 Anler Hernández <hello@anler.me>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import uuid
import threading

from django.conf import settings

import factory


class Factory(factory.DjangoModelFactory):
    class Meta:
        strategy = factory.CREATE_STRATEGY
        model = None
        abstract = True

    _SEQUENCE = 1
    _SEQUENCE_LOCK = threading.Lock()

    @classmethod
    def _setup_next_sequence(cls):
        with cls._SEQUENCE_LOCK:
            cls._SEQUENCE += 1
        return cls._SEQUENCE


class ProjectTemplateFactory(Factory):
    class Meta:
        strategy = factory.CREATE_STRATEGY
        model = "projects.ProjectTemplate"
        django_get_or_create = ("slug",)

    name = "Template name"
    slug = settings.DEFAULT_PROJECT_TEMPLATE
    description = factory.Sequence(lambda n: "Description {}".format(n))

    us_statuses = []
    points = []
    task_statuses = []
    issue_statuses = []
    issue_types = []
    priorities = []
    severities = []
    roles = []
    default_owner_role = "tester"


class ProjectFactory(Factory):
    class Meta:
        model = "projects.Project"
        strategy = factory.CREATE_STRATEGY

    name = factory.Sequence(lambda n: "Project {}".format(n))
    slug = factory.Sequence(lambda n: "project-{}-slug".format(n))
    description = "Project description"
    owner = factory.SubFactory("tests.factories.UserFactory")
    creation_template = factory.SubFactory("tests.factories.ProjectTemplateFactory")


class RoleFactory(Factory):
    class Meta:
        model = "users.Role"
        strategy = factory.CREATE_STRATEGY

    name = factory.Sequence(lambda n: "Role {}".format(n))
    slug = factory.Sequence(lambda n: "test-role-{}".format(n))
    project = factory.SubFactory("tests.factories.ProjectFactory")


class UserFactory(Factory):
    class Meta:
        model = "users.User"
        strategy = factory.CREATE_STRATEGY

    username = factory.Sequence(lambda n: "user{}".format(n))
    email = factory.LazyAttribute(lambda obj: '%s@email.com' % obj.username)
    password = factory.PostGeneration(lambda obj, *args, **kwargs: obj.set_password(obj.username))


class MembershipFactory(Factory):
    class Meta:
        model = "projects.Membership"
        strategy = factory.CREATE_STRATEGY

    token = factory.LazyAttribute(lambda obj: str(uuid.uuid1()))
    project = factory.SubFactory("tests.factories.ProjectFactory")
    role = factory.SubFactory("tests.factories.RoleFactory")
    user = factory.SubFactory("tests.factories.UserFactory")



# Copyright (C) 2014-2016 Andrey Antukh <niwi@niwi.nz>
# Copyright (C) 2014-2016 Jesús Espino <jespinog@gmail.com>
# Copyright (C) 2014-2016 David Barragán <bameda@dbarragan.com>
# Copyright (C) 2014-2016 Alejandro Alonso <alejandro.alonso@kaleidos.net>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import requests
import json
import re 

from pprint import pprint

from collections import namedtuple
from urllib.parse import urljoin

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from taiga.base.connectors.exceptions import ConnectorBaseException


class GitLabApiError(ConnectorBaseException):
    pass

######################################################
## Data
######################################################
GITLAB_URL = getattr(settings, "GITLAB_URL", None)
GITLAB_APP_ID = getattr(settings, "GITLAB_APP_ID", None)
GITLAB_APP_SECRET = getattr(settings, "GITLAB_APP_SECRET", None)

API_RESOURCES_URLS = {
    "login": {
        "authorize": "oauth/authorize",
        "access-token": "oauth/token"
    },
    "user": {
        "profile": "api/v3/user"
    }
}

HEADERS = {"Accept": "application/json",}

AuthInfo = namedtuple("AuthInfo", ["access_token"])
User = namedtuple("User", ["id", "username", "full_name", "bio", "email"])
Email = namedtuple("Email", ["email", "is_primary"])


######################################################
## utils
######################################################

def _build_url(*args, **kwargs) -> str:
    """
    Return a valid url.
    """
    resource_url = API_RESOURCES_URLS
    for key in args:
        resource_url = resource_url[key]

    if kwargs:
        resource_url = resource_url.format(**kwargs)

    return urljoin(GITLAB_URL, resource_url)


def _get(url:str, headers:dict) -> dict:
    """
    Make a GET call.
    """
    response = requests.get(url, headers=headers)

    data = response.json()
    if response.status_code != 200:
        raise GitLabApiError({"status_code": response.status_code,
                                  "error": data.get("message", response.text)})
    return data

def _post(url:str, params:dict, headers:dict) -> dict:
    """
    Make a POST call.
    """

    preparedRequest = requests.Request('POST', url, files=params, headers=headers).prepare()
    response = requests.Session().send(preparedRequest)

    data = response.json()
    if response.status_code != 200 or "error" in data:
        raise GitLabApiError({"status_code": response.status_code,
                                  "error": data.get("error_description", response.text)})
    return data


######################################################
## Simple calls
######################################################

def login(access_code:str, redirect_uri:str, application_id:str=GITLAB_APP_ID, application_secret:str=GITLAB_APP_SECRET,
          headers:dict=HEADERS):
    """
    Get access_token fron an user authorized code, the GitLab application id and secret key.
    (See https://github.com/doorkeeper-gem/doorkeeper - the middleware used by  GitLab).
    """

    if not application_id:
        raise GitLabApiError({"error_message": _("Missing GITLAB_APP_ID from configuration.")})

    if not application_secret:
        raise GitLabApiError({"error_message": _("Missing GITLAB_APP_SECRET from configuration.")})

    if not GITLAB_URL:
        raise GitLabApiError({"error_message": _("Missing GITLAB_URL from configuration.")})

    authorized_redirect = re.sub(r'(?is)&code=.+', '', redirect_uri)

    url = urljoin(GITLAB_URL, "oauth/token")
    print("OAUTH: ", url, " - code: ", access_code, " - client id: ", application_id, "- secret: ", application_secret)
    params={"code": (None, access_code),
            "client_id": (None, application_id),
            "client_secret": (None, application_secret),
	    "grant_type": (None, "authorization_code"),
            "redirect_uri": (None, authorized_redirect)
	   }
    data = _post(url, params=params, headers=headers)
    return AuthInfo(access_token=data.get("access_token", None))


def get_user_profile(headers:dict=HEADERS):
    """
    Get authenticated user info.
    (See http://doc.gitlab.com/ce/api/users.html#for-normal-users).
    """
    url = _build_url("user", "profile")
    data = _get(url, headers=headers)
    return User(id=data.get("id", None),
		email=data.get("email", None),
                username=data.get("login", data.get("username", None)),
                full_name=(data.get("name", None) or ""),
                bio=(data.get("bio", None) or ""))


######################################################
## Convined calls
######################################################

def me(access_code:str, redirect_uri:str) -> tuple:
    """
    Connect to a gitlab account and get all personal info (profile and the primary email).
    """
    auth_info = login(access_code, redirect_uri)

    headers = HEADERS.copy()
    headers["Authorization"] = "Bearer {}".format(auth_info.access_token)

    print ("Authorization: ", headers["Authorization"]);
    user = get_user_profile(headers=headers)
    return user.email, user




# Copyright (C) 2014-2016 Andrey Antukh <niwi@niwi.nz>
# Copyright (C) 2014-2016 Jesús Espino <jespinog@gmail.com>
# Copyright (C) 2014-2016 David Barragán <bameda@dbarragan.com>
# Copyright (C) 2014-2016 Alejandro Alonso <alejandro.alonso@kaleidos.net>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.db import transaction as tx
from django.db import IntegrityError
from django.utils.translation import ugettext as _

from django.apps import apps

from taiga.base.utils.slug import slugify_uniquely
from taiga.base import exceptions as exc
from taiga.auth.services import send_register_email
from taiga.auth.services import make_auth_response_data, get_membership_by_token
from taiga.auth.signals import user_registered as user_registered_signal

from . import connector


@tx.atomic
def gitlab_register(username:str, email:str, full_name:str, gitlab_id:int, bio:str):
    """
    Register a new user from gitlab.

    This can raise `exc.IntegrityError` exceptions in
    case of conflics found.

    :returns: User
    """
    auth_data_model = apps.get_model("users", "AuthData")
    user_model = apps.get_model("users", "User")

    try:
        # Gitlab user association exist?
        auth_data = auth_data_model.objects.get(key="gitlab", value=gitlab_id)
        user = auth_data.user
    except auth_data_model.DoesNotExist:
        try:
            # Is a user with the same email as the gitlab user?
            user = user_model.objects.get(email=email)
            auth_data_model.objects.create(user=user, key="gitlab", value=gitlab_id, extra={})
        except user_model.DoesNotExist:
            # Create a new user
            username_unique = slugify_uniquely(username, user_model, slugfield="username")
            user = user_model.objects.create(email=email,
                                             username=username_unique,
                                             full_name=full_name,
                                             bio=bio)
            auth_data_model.objects.create(user=user, key="gitlab", value=gitlab_id, extra={})

            send_register_email(user)
            user_registered_signal.send(sender=user.__class__, user=user)

    return user


def gitlab_login_func(request):
    code = request.DATA.get('code', None)
    redirect_uri = request.DATA.get('redirect_uri', None)

    email, user_info = connector.me(code, redirect_uri)

    user = gitlab_register(username=user_info.username,
                           email=email,
                           full_name=user_info.full_name,
                           gitlab_id=user_info.id,
                           bio=user_info.bio)

    data = make_auth_response_data(user)
    return data



# Copyright (C) 2014 Andrey Antukh <niwi@niwi.be>
# Copyright (C) 2014 Jesús Espino <jespinog@gmail.com>
# Copyright (C) 2014 David Barragán <bameda@dbarragan.com>
# Copyright (C) 2014 Anler Hernández <hello@anler.me>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest

from unittest.mock import patch, Mock
from taiga_contrib_gitlab_auth import connector as gitlab


def test_url_builder():
    assert (gitlab._build_url("https://test.gitlab.ts", "login", "authorize") ==
            "https://test.gitlab.ts/login/oauth/authorize")
    assert (gitlab._build_url("https://test.gitlab.ts", "login","access-token") ==
            "https://test.gitlab.ts/login/oauth/access_token")
    assert (gitlab._build_url("https://test.gitlab.ts", "user", "profile") ==
            "https://test.gitlab.ts/user")
    assert (gitlab._build_url("https://test.gitlab.ts", "user", "emails") ==
            "https://test.gitlab.ts/user/emails")


def test_login_without_settings_params():
    with pytest.raises(gitlab.GitLabApiError) as e, \
            patch("taiga_contrib_gitlab_auth.connector.requests") as m_requests:
        m_requests.post.return_value = m_response = Mock()
        m_response.status_code = 200
        m_response.json.return_value = {"access_token": "xxxxxxxx"}

        auth_info = gitlab.login("*access-code*", "**client-id**", "*ient-secret*", gitlab.HEADERS)
    assert e.value.status_code == 400
    assert "error_message" in e.value.detail


def test_login_success():
    with patch("taiga_contrib_gitlab_auth.connector.requests") as m_requests, \
            patch("taiga_contrib_gitlab_auth.connector.CLIENT_ID") as CLIENT_ID, \
            patch("taiga_contrib_gitlab_auth.connector.CLIENT_SECRET") as CLIENT_SECRET:
        CLIENT_ID = "*CLIENT_ID*"
        CLIENT_SECRET = "*CLIENT_SECRET*"
        m_requests.post.return_value = m_response = Mock()
        m_response.status_code = 200
        m_response.json.return_value = {"access_token": "xxxxxxxx"}

        auth_info = gitlab.login("*access-code*", "**client-id**", "*client-secret*", gitlab.HEADERS)

        assert auth_info.access_token == "xxxxxxxx"
        m_requests.post.assert_called_once_with("https://gitlab.test.ts/login/oauth/access_token",
                                                headers=gitilab.HEADERS,
                                                params={'code': '*access-code*',
                                                        'scope': 'user:emails',
                                                        'client_id': '**client-id**',
                                                        'client_secret': '*client-secret*'})


def test_login_whit_errors():
    with pytest.raises(gitlab.GitLabApiError) as e, \
            patch("taiga_contrib_gitlab_auth.connector.requests") as m_requests, \
            patch("taiga_contrib_gitlab_auth.connector.CLIENT_ID") as CLIENT_ID, \
            patch("taiga_contrib_gitlab_auth.connector.CLIENT_SECRET") as CLIENT_SECRET:
        CLIENT_ID = "*CLIENT_ID*"
        CLIENT_SECRET = "*CLIENT_SECRET*"
        m_requests.post.return_value = m_response = Mock()
        m_response.status_code = 200
        m_response.json.return_value = {"error": "Invalid credentials"}

        auth_info = gitlab.login("*access-code*", "**client-id**", "*ient-secret*", gitlab.HEADERS)
    assert e.value.status_code == 400
    assert e.value.detail["status_code"] == 200
    assert e.value.detail["error"] == "Invalid credentials"


def test_get_user_profile_success():
    with patch("taiga_contrib_gitlab_auth.connector.requests") as m_requests:
        m_requests.get.return_value = m_response = Mock()
        m_response.status_code = 200
        m_response.json.return_value = {"id": 1955,
                                        "login": "mmcfly",
                                        "name": "martin seamus mcfly",
                                        "bio": "time traveler"}

        user_profile = gitlab.get_user_profile(gitlab.HEADERS)

        assert user_profile.id == 1955
        assert user_profile.username == "mmcfly"
        assert user_profile.full_name == "martin seamus mcfly"
        assert user_profile.bio == "time traveler"
        m_requests.get.assert_called_once_with("https://api.gitlab.com/user",
                                               headers=gitlab.HEADERS)


def test_get_user_profile_whit_errors():
    with pytest.raises(gitlab.GitLabApiError) as e, \
            patch("taiga_contrib_gitlab_auth.connector.requests") as m_requests:
        m_requests.get.return_value = m_response = Mock()
        m_response.status_code = 401
        m_response.json.return_value = {"error": "Invalid credentials"}

        auth_info = gitlab.get_user_profile(gitlab.HEADERS)
    assert e.value.status_code == 400
    assert e.value.detail["status_code"] == 401
    assert e.value.detail["error"] == "Invalid credentials"


def test_get_user_emails_success():
    with patch("taiga_contrib_gitlab_auth.connector.requests") as m_requests:
        m_requests.get.return_value = m_response = Mock()
        m_response.status_code = 200
        m_response.json.return_value = [{"email": "darth-vader@bttf.com", "primary": False},
                                        {"email": "mmcfly@bttf.com", "primary": True}]

        emails = gitlab.get_user_emails(gitlab.HEADERS)

        assert len(emails) == 2
        assert emails[0].email == "darth-vader@bttf.com"
        assert not emails[0].is_primary
        assert emails[1].email == "mmcfly@bttf.com"
        assert emails[1].is_primary
        m_requests.get.assert_called_once_with("https://api.gitlab.com/user/emails",
                                               headers=gitlab.HEADERS)


def test_get_user_emails_whit_errors():
    with pytest.raises(gitlab.GitLabApiError) as e, \
            patch("taiga_contrib_gitlab_auth.connector.requests") as m_requests:
        m_requests.get.return_value = m_response = Mock()
        m_response.status_code = 401
        m_response.json.return_value = {"error": "Invalid credentials"}

        emails = gitlab.get_user_emails(gitlab.HEADERS)
    assert e.value.status_code == 400
    assert e.value.detail["status_code"] == 401
    assert e.value.detail["error"] == "Invalid credentials"


def test_me():
    with patch("taiga_contrib_gitlab_auth.connector.login") as m_login, \
            patch("taiga_contrib_gitlab_auth.connector.get_user_profile") as m_get_user_profile, \
            patch("taiga_contrib_gitlab_auth.connector.get_user_emails") as m_get_user_emails:
        m_login.return_value = gitlab.AuthInfo(access_token="xxxxxxxx")
        m_get_user_profile.return_value = gitlab.User(id=1955,
                                                      username="mmcfly",
                                                      full_name="martin seamus mcfly",
                                                      bio="time traveler")
        m_get_user_emails.return_value = [gitlab.Email(email="darth-vader@bttf.com", is_primary=False),
                                          gitlab.Email(email="mmcfly@bttf.com", is_primary=True)]

        email, user = gitlab.me("**access-code**")

        assert email == "mmcfly@bttf.com"
        assert user.id == 1955
        assert user.username == "mmcfly"
        assert user.full_name == "martin seamus mcfly"
        assert user.bio == "time traveler"

        headers = gitlab.HEADERS.copy()
        headers["Authorization"] = "token xxxxxxxx"
        m_get_user_profile.assert_called_once_with(headers=headers)
        m_get_user_emails.assert_called_once_with(headers=headers)



# Copyright (C) 2014 Andrey Antukh <niwi@niwi.be>
# Copyright (C) 2014 Jesús Espino <jespinog@gmail.com>
# Copyright (C) 2014 David Barragán <bameda@dbarragan.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from settings.development import *

SKIP_SOUTH_TESTS = True
SOUTH_TESTS_MIGRATE = False
CELERY_ALWAYS_EAGER = True
CELERY_ENABLED = False

MEDIA_ROOT = "/tmp"

EMAIL_BACKEND = "django.core.mail.backends.locmem.EmailBackend"
INSTALLED_APPS = INSTALLED_APPS + [
    "taiga_contrib_gitlab_auth",
]
INSTALLED_APPS = set(INSTALLED_APPS) - set(["taiga.hooks.github", "taiga.hooks.gitlab", "taiga.hooks.bitbucket"])

REST_FRAMEWORK["DEFAULT_THROTTLE_RATES"] = {
    "anon": None,
    "user": None,
    "import-mode": None,
    "import-dump-mode": None,
}



# Copyright (C) 2014 Andrey Antukh <niwi@niwi.be>
# Copyright (C) 2014 Jesús Espino <jespinog@gmail.com>
# Copyright (C) 2014 David Barragán <bameda@dbarragan.com>
# Copyright (C) 2014 Anler Hernández <hello@anler.me>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest
from unittest.mock import patch

from django.apps import apps
from django.core.urlresolvers import reverse

from .. import factories

from taiga_contrib_gitlab_auth import connector as gitlab_connector

pytestmark = pytest.mark.django_db


def test_response_200_in_registration_with_gitlab_account(client, settings):
    settings.PUBLIC_REGISTER_ENABLED = False
    form = {"type": "gitlab",
            "code": "xxxxxx"}

    auth_data_model = apps.get_model("users", "AuthData")

    with patch("taiga_contrib_gitlab_auth.connector.me") as m_me:
        m_me.return_value = ("mmcfly@bttf.com",
                             gitlab_connector.User(id=1955,
                                                   username="mmcfly",
                                                   full_name="martin seamus mcfly",
                                                   bio="time traveler"))

        response = client.post(reverse("auth-list"), form)
        assert response.status_code == 200
        assert response.data["username"] == "mmcfly"
        assert response.data["auth_token"] != "" and response.data["auth_token"] is not None
        assert response.data["email"] == "mmcfly@bttf.com"
        assert response.data["full_name"] == "martin seamus mcfly"
        assert response.data["bio"] == "time traveler"
        assert auth_data_model.objects.filter(user__username="mmcfly", key="github", value="1955").count() == 1


def test_response_200_in_registration_with_gitlab_account_and_existed_user_by_email(client, settings):
    settings.PUBLIC_REGISTER_ENABLED = False
    form = {"type": "gitlab",
            "code": "xxxxxx"}
    user = factories.UserFactory()
    user.email = "mmcfly@bttf.com"
    user.save()

    with patch("taiga_contrib_gitlab_auth.connector.me") as m_me:
        m_me.return_value = ("mmcfly@bttf.com",
                             gitlab_connector.User(id=1955,
                                                   username="mmcfly",
                                                   full_name="martin seamus mcfly",
                                                   bio="time traveler"))

        response = client.post(reverse("auth-list"), form)
        assert response.status_code == 200
        assert response.data["username"] == user.username
        assert response.data["auth_token"] != "" and response.data["auth_token"] is not None
        assert response.data["email"] == user.email
        assert response.data["full_name"] == user.full_name
        assert response.data["bio"] == user.bio
        assert user.auth_data.filter(key="github", value="1955").count() == 1


def test_response_200_in_registration_with_gitlab_account_and_existed_user_by_gitlab_id(client, settings):
    settings.PUBLIC_REGISTER_ENABLED = False
    form = {"type": "gitlab",
            "code": "xxxxxx"}
    user = factories.UserFactory.create()

    auth_data_model = apps.get_model("users", "AuthData")
    auth_data_model.objects.create(user=user, key="gitlab", value="1955", extra={})

    with patch("taiga_contrib_gitlab_auth.connector.me") as m_me:
        m_me.return_value = ("mmcfly@bttf.com",
                             gitlab_connector.User(id=1955,
                                                   username="mmcfly",
                                                   full_name="martin seamus mcfly",
                                                   bio="time traveler"))

        response = client.post(reverse("auth-list"), form)
        assert response.status_code == 200
        assert response.data["username"] != "mmcfly"
        assert response.data["auth_token"] != "" and response.data["auth_token"] is not None
        assert response.data["email"] != "mmcfly@bttf.com"
        assert response.data["full_name"] != "martin seamus mcfly"
        assert response.data["bio"] != "time traveler"


def test_response_200_in_registration_with_gitlab_account_and_change_gitlab_username(client, settings):
    settings.PUBLIC_REGISTER_ENABLED = False
    form = {"type": "gitlab",
            "code": "xxxxxx"}
    user = factories.UserFactory()
    user.username = "mmcfly"
    user.save()

    auth_data_model = apps.get_model("users", "AuthData")

    with patch("taiga_contrib_gitlab_auth.connector.me") as m_me:
        m_me.return_value = ("mmcfly@bttf.com",
                             gitlab_connector.User(id=1955,
                                                   username="mmcfly",
                                                   full_name="martin seamus mcfly",
                                                   bio="time traveler"))

        response = client.post(reverse("auth-list"), form)
        assert response.status_code == 200
        assert response.data["username"] == "mmcfly-1"
        assert response.data["auth_token"] != "" and response.data["auth_token"] is not None
        assert response.data["email"] == "mmcfly@bttf.com"
        assert response.data["full_name"] == "martin seamus mcfly"
        assert response.data["bio"] == "time traveler"
        assert auth_data_model.objects.filter(user__username="mmcfly-1", key="github", value="1955").count() == 1


def test_response_200_in_registration_with_gitlab_account_in_a_project(client, settings):
    settings.PUBLIC_REGISTER_ENABLED = False
    membership_model = apps.get_model("projects", "Membership")
    membership = factories.MembershipFactory(user=None)
    form = {"type": "gitlab",
            "code": "xxxxxx",
            "token": membership.token}

    with patch("taiga_contrib_gitlab_auth.connector.me") as m_me:
        m_me.return_value = ("mmcfly@bttf.com",
                             github_connector.User(id=1955,
                                                   username="mmcfly",
                                                   full_name="martin seamus mcfly",
                                                   bio="time traveler"))

        response = client.post(reverse("auth-list"), form)
        assert response.status_code == 200
        assert membership_model.objects.get(token=form["token"]).user.username == "mmcfly"


def test_response_404_in_registration_with_gitlab_in_a_project_with_invalid_token(client, settings):
    settings.PUBLIC_REGISTER_ENABLED = False
    form = {"type": "gitlab",
            "code": "xxxxxx",
            "token": "123456"}

    with patch("taiga_contrib_gitlab_auth.connector.me") as m_me:
        m_me.return_value = ("mmcfly@bttf.com",
                             gitlab_connector.User(id=1955,
                                                   username="mmcfly",
                                                   full_name="martin seamus mcfly",
                                                   bio="time traveler"))

        response = client.post(reverse("auth-list"), form)
        assert response.status_code == 404



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from taiga_contrib_gitlab_auth import __version__
from setuptools import setup, find_packages

setup(
    name = 'taiga-contrib-gitlab-auth',
    version = '.'.join(map(str,__version__)),
    description = "The Taiga plugin for gitlab authentication",
    long_description = "This plugin allows integration with a GitLab installation",
    keywords = 'taiga, gitlab, auth, plugin',
    author = 'Fabio "MrWHO" Torchetti',
    author_email = 'mrwho@wedjaa.net',
    url = 'https://github.com/WedjaaOpen/taiga-contrib-gitlab-auth',
    download_url = 'https://github.com/WedjaaOpen/taiga-contrib-gitlab-auth/tarball/' + '.'.join(map(str,__version__)),
    license = 'AGPL',
    include_package_data = True,
    packages = find_packages(),
    install_requires=[],
    setup_requires = [],
    classifiers = [
        "Programming Language :: Python",
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ]
)
